@@ -392,6 +392,17 @@ class material:
                         continue
                     yield image.filepath
 
+    @staticmethod
+    def eachEnalbeTexture(m):
+        for i, slot in enumerate(m.texture_slots):
+            if m.use_textures[i] and slot and slot.texture:
+                texture=slot.texture
+                if  texture.type=="IMAGE":
+                    image=texture.image
+                    if not image:
+                        continue
+                    yield slot.texture
+
 
 class mesh:
     @staticmethod
@@ -506,8 +517,13 @@ class mesh:
         enterObjectMode()
 
     @staticmethod
-    def flipNormals(m):
-        m.flipNormals()
+    def flipNormals(mesh_object):
+        bpy.ops.object.select_all(action='DESELECT')
+        object.activate(mesh_object)
+        # edit
+        enterEditMode()
+        bpy.ops.mesh.flip_normals()
+        enterObjectMode()
 
     @staticmethod
     def addMaterial(m, material):
@@ -578,6 +594,10 @@ class face:
     def setSmooth(face, isSmooth):
         face.use_smooth=True if isSmooth else False
 
+    @staticmethod
+    def isSmooth(face):
+        return face.use_smooth
+
 
 class armature:
     @staticmethod
@@ -677,10 +697,34 @@ class constraint:
     def isIKSolver(c):
         return c.type=='IK'
 
+    @staticmethod
+    def isCopyRotation(c):
+        return c.type=='COPY_ROTATION'
+
+    @staticmethod
+    def addCopyRotation(pose_bone, target_object, target_bone, factor):
+        c=pose_bone.constraints.new(type='COPY_ROTATION')
+        c.target=target_object
+        c.subtarget=target_bone.name
+        c.influence=factor
+        c.target_space='LOCAL'
+        c.owner_space='LOCAL'
+
+
+"""
+custom property keys
+"""
 MMD_SHAPE_GROUP_NAME='_MMD_SHAPE'
+
 MMD_MB_NAME='mb_name'
+MMD_ENGLISH_NAME='english_name'
+
 MMD_MB_COMMENT='mb_comment'
 MMD_COMMENT='comment'
+MMD_ENGLISH_COMMENT='english_comment'
+
+BONE_ENGLISH_NAME='english_name'
+
 BASE_SHAPE_NAME='Basis'
 RIGID_NAME='rigid_name'
 RIGID_SHAPE_TYPE='rigid_shape_type'
@@ -704,3 +748,12 @@ CONSTRAINT_SPRING_POS='const_spring_pos'
 CONSTRAINT_SPRING_ROT='const_spring_rot'
 TOON_TEXTURE_OBJECT='ToonTextures'
 
+MATERIALFLAG_BOTHFACE='material_flag_bothface'
+MATERIALFLAG_GROUNDSHADOW='material_flag_groundshadow'
+MATERIALFLAG_SELFSHADOWMAP='material_flag_selfshadowmap'
+MATERIALFLAG_SELFSHADOW='material_flag_drawselfshadow'
+MATERIALFLAG_EDGE='material_flag_drawedge'
+MATERIAL_SHAREDTOON='material_shared_toon'
+MATERIAL_SPHERE_MODE='material_sphere_mode'
+TEXTURE_TYPE='texture_type'
+
